////////////////////////////////////////
/**************************************/
/*                                    */
/* Michael Goldbuch; Jrgen Dffinger */
/*                                    */
/* Mikroprozessortechnik              */
/* Praktikum 5 / Aufgabe1             */
/* Version 1.0                        */
/* 2011-05-17                         */
/*                                    */
/**************************************/
////////////////////////////////////////



//	definitions for ATMega8
.include "m8def.inc"



/***************   Interupts   ***************/

rjmp RESET ; Reset Handler
reti; rjmp EXT_INT0 ; IRQ0 Handler
rjmp EXT_INT1 ; IRQ1 Handler
reti; rjmp TIM2_COMP ; Timer2 Compare Handler
reti; rjmp TIM2_OVF ; Timer2 Overflow Handler
reti; rjmp TIM1_CAPT ; Timer1 Capture Handler
reti; rjmp TIM1_COMPA ; Timer1 CompareA Handler
reti; rjmp TIM1_COMPB ; Timer1 CompareB Handler
reti; rjmp TIM1_OVF ; Timer1 Overflow Handler
reti; rjmp TIM0_OVF ; Timer0 Overflow Handler
reti; rjmp SPI_STC ; SPI Transfer Complete Handler
reti; rjmp USART_RXC ; USART RX Complete Handler
reti; rjmp USART_UDRE ; UDR Empty Handler
reti; rjmp USART_TXC ; USART TX Complete Handler
reti; rjmp ADCI ; ADC Conversion Complete Handler
reti; rjmp EE_RDY ; EEPROM Ready Handler
reti; rjmp ANA_COMP ; Analog Comparator Handler
reti; rjmp TWSI ; Two-wire Serial Interface Handler
reti; rjmp SPM_RDY ; Store Program Memory Ready Handler



/***************   Includes   ***************/

.include "motor.asm"			//	Funktionen zur Steuerung der Motoren
.include "arithmetic.asm"		//	arithmetische Funktionen
.include "USART.asm"			//	Funktionen zur Kommunikation ber USART
.include "adc.asm"				//	Funktionen zum ADC
.include "functions.asm"		//	allgemeine Funktionen einbinden
.include "extintfunc.asm"		//	externe Interrupt Funktionen
.include "led.asm"				//	Funktionen zur Manipulation der LEDs




/***************   RESET   ***************/

RESET:

	// 	Stack initialisieren
	ldi R16,high(RAMEND)
	out sph,R16
	ldi R16,low(RAMEND)	
	out spl,R16		


	// 	USART initialisieren
	rcall USART_INIT

	//	hintere LEDs initialisieren (abschalten)
	sbi DDRD,7
	sbi DDRC,0
	sbi DDRC,1
	cbi PORTD,7
	cbi PORTC,0
	cbi PORTC,1

	//	PWM initialisieren
	in  R16,DDRB
	ori R16,0b00110110
	out DDRB,R16
	in  R16,DDRD
	ori R16,0b00110000
	out DDRD,R16
	sbi PORTB,1
	sbi PORTB,2

	//	PWM initialisieren
	rcall PWM_INIT
	
	//	ADC initialisieren
	rcall ADC_INIT

	//	ADC4 (switches) enable
	rcall ADC4_Enable
	
	//	EXT_INT_INIT
	rcall EXT_INT_INIT

	//	Vorwrtsfahrt initiiern
	//rcall drive_forward

	//	hintere LEDs abschalten
	BackLED 0x00

	//	Ergebnisregister R10 und Interrupt-Flag-Register R11 lschen
	clr R10
	clr R11

	// global interupt enable
	sei

rjmp main



/***************   Hauptprogramm   ***************/

main:

	//	PD3 auf High-Level setzen (fr Spannungsteiler)
	sbi DDRD,3
	sbi PORTD,3

	//	Prfen ob ein Interrupt ausgelst wurde (R11 > 0)
	mov  R16,R11
	tst  R16
	breq main	//	Wenn nicht dann weiter mit Schleife

	//	externen Interrupt 1 abschalten
	in   R16,GICR
	andi R16,0b01111111
	out  GICR,R16
		

	//	Ergebnis der Messung welcher Taster gesetzt ist ber USART ausgeben
	clr R18
	mov R19,R10
//	TX_BIN2ASCII R18,R19

	
	//	Prfen ob einer der linken oder rechten Schalter gedrckt wurde
	mov  R16,R10
	cpi  R16,0x06
	brlo main_1
	
	//	Wenn einer der linken Schalter gedrckt wurde, dann rechte Kurve initiieren 
	//rcall right_curve
	rjmp main_2

main_1:

	//	Wenn einer der rechten Schaalter gedrckt wurde, dann linke Kurve initiieren
	//rcall left_curve

main_2:

	//	hintere LEDs einschalten, zur Anzeige Rckwrtsfahrt
	BackLED 0x00

	//	Ergebnisregister R10 und Interrupt-Flag-Register R11 lschen
	clr R10
	clr R11

	//	Wartezeit fr Rckwrtsfahrt 
	counter 0x0F,0x00,0x00

	//	Vorwrtsfahrt initieren
	//rcall drive_forward
	BackLED 0x00

	//	Wartezeit (Habe ich reingenommen, da bei linker Kurve das Problem auftauchte,
	//	dass scheinbar ein weiterer Interrrupt ausgelst wurde, welcher eine Rechtskurve 
	//	auslste. Vermutlich kommt dies vom Motor, so wie es in manchen Dokumentationen
	//	beschrieben wird.)
	//counter 0x1F,0x00,0x00

	//	externen Interrupt 1 einschalten
	in   R16,GICR
	ori  R16,0b10000000
	out  GICR,R16

rjmp main




/*************** EXT_INT1   ***************/

EXT_INT1:	
	
	//	Register und Statusregister sichern
	push R16
	in   R16,SREG
	push R16
	push R18
	push R19

	//	PD3 auf High-Level setzen (fr Spannungsteiler)
	sbi DDRD,3
	sbi PORTD,3

	//	Messung der Schalterspannung durchfhren
	rcall ADC_Conversion
	
	//	Zwischenergebnis = (1024 * 64) / Messergebnis 
	ldi R19,0xFF
	ldi R18,0xFF
	DIV16 R19,R18,R0,R1

	//	Ergebnis = Zwischenergebnis - 64 
	mov  R18,R0
	mov  R19,R1
	clr  R16
	subi R19,0x40
	sbc  R18,R16


	//	Ergebnis asugeben
	TX_Bin2ASCII R18,R19

	
	//	Ergebnis in Ergebnisregister R10 speichern und durch inkrementieren von R11 anzeigen, dass 
	//  der externe Interrupt 1 ausgelst wurde (R11 > 0)
	mov R10,R19
	inc R11	

	//	PD7 wieder als Eingang initialisieren 
	cbi PORTD,3
	cbi DDRD,3

	//	externen Interrupt 1 abschalten
	in   R16,GIFR
	andi R16,0b01111111
	out  GIFR,R16

	//	Register und Statusregister wiederherstellen
	pop  R19
	pop  R18
	pop  R16
	out  SREG,R16
	pop  R16

reti



/***************   right_curve   ***************/

right_curve:
	motorspeed 0xAF,0xAF	
	rcall MotorDir_RWD_RWD
	counter 0x07,0x00,0x00
	rcall MotorDir_FWD_RWD

ret



/***************   left_curve   ***************/

left_curve:
	motorspeed 0xAF,0xAF	
	rcall MotorDir_RWD_RWD
	counter 0x07,0x00,0x00
	rcall MotorDir_RWD_FWD

ret



/***************   drive_forward   ***************/

drive_forward:

	rcall Motor_Stop
	rcall MotorDir_FWD_FWD
	motorspeed 0xAF,0xAF

ret





