//////////////////////////////////////////////////////////
/********************************************************/
/*                                                      */
/*  Fachhochschule Jena                                 */
/*  Mikroprozessortechnik                               */
/*  Michael Goldbuch; Jrgen Dffinger                  */
/*                                                      */
/*  arithmetic.asm   Version 1.0.0   2011-05-17         */
/*                                                      */
/*  Die arithmetic.asm enthlt arithmetische Funktionen */
/*                                                      */
/********************************************************/
//////////////////////////////////////////////////////////




/***************   Division 16 bit = 16 bit / 8 bit   ***************/

//	Achtung die Ergebnisse werden in die Register R0 (Low Byte Quotient),
//	R1 (High Byte Quotient) und R2 (Rest) geschrieben, daher hat die
//	aufrufende Funktion diese Register selber zu sichern.

.macro DIV

	//	Register sichern
	push R16
	push R17
	push R18
	
	//	Register initialisieren
	mov R16,@0		//	Dividend (High Byte)
	mov R17,@1  	//	Dividend (Low Byte)
	mov R18,@2		//	Divisor

	//	Division durchfhren
	rcall div_exec	

	//	Register wiederherstellen
	pop R18
	pop R17
	pop R16

.endm



div_exec:

	//	Register sichern
	push R22
	push R23
	
	//	Initialisierung 
	clr R0			//	Quotient (High Byte) auf null setzen
	clr R1			//	Quotient (Low Byte) auf null setzen
	clr R2			//	Rest auf null setzen
	ldi R22,0x10	//	Zhler auf 16 zu berechnende bit Werte einstellen

div_exec_1:

	lsl  R17        //	hchstes Bit des Low Bytes des Dividenden ins Carry Flag schieben
	rol  R16		//	hchstes Bit des Dividenden in Carry Flag schieben und aus Carry bernehmen
	rol  R2 		//	Wert im Carry Flag ins niederwertigste Bit des Rests schieben
	cp   R2,R18	    //	Rest mit Divisor vergleichen
	brcs div_exec_2
	
	//	Wenn Rest grer als Divisor...
	sub  R2,R18	//	Rest = Rest - Divisor
	sec				//	Carry Flag setzen
	rol	 R1		//	Mittels des gesetzten Carry Flags eine 1 in das niederwertigste Bit des Quotienten schieben
	rol  R0
	rjmp div_exec_3

div_exec_2:
	
	//	Wenn Rest kleiner als Divisor...
	lsl  R1		//	eine null in das niederwertigste Bit des Quotienten schieben
	rol  R0

div_exec_3:
		
	dec	 R22		//	Zhler dekrementieren
	cpi	 R22,0x00	//	Zhler prfen ob bereits alle Berechnungen durchgefhrt wurden
	brne div_exec_1	//	Wenn nein, dann weiter mit Berechnungen
	
	//	Register wiederherstellen
	pop R23
	pop R22

ret



/***************   Division 16 bit = 16 bit / 16 bit   ***************/

//	Achtung die Ergebnisse werden in die Register R0 (Low Byte Quotient),
//	R1 (High Byte Quotient) und R2 (Rest) geschrieben, daher hat die
//	aufrufende Funktion diese Register selber zu sichern.

.macro DIV16

	//	Register sichern
	push R16
	push R17
	push R18
	push R19
	
	//	Register initialisieren
	mov R16,@0		//	Dividend (High Byte)
	mov R17,@1  	//	Dividend (Low Byte)
	mov R18,@2		//	Divisor  (High Byte)
	mov R19,@3	    //	Divisor  (Low Byte)

	//	Division durchfhren
	rcall div16_exec	

	//	Register wiederherstellen
	pop R19
	pop R18
	pop R17
	pop R16

.endm



div16_exec:

	//	Register sichern
	push R22
	push R23
	
	//	Initialisierung 
	clr R0			//	Quotient (High Byte) auf null setzen
	clr R1			//	Quotient (Low Byte) auf null setzen
	clr R2			//	Rest (High Byte) auf null setzen
	clr R3			//	Rest (Low Byte) auf null setzen
	ldi R22,0x10	//	Zhler auf 16 zu berechnende bit Werte einstellen

div16_exec_1:

	lsl  R17        	//	hchstes Bit des Low Bytes des Dividenden ins Carry Flag schieben
	rol  R16			//	hchstes Bit des Dividenden in Carry Flag schieben und aus Carry bernehmen
	rol  R3 			//	Wert im Carry Flag ins niederwertigste Bit des Rests schieben
	rol  R2				//	bertrag in das High Byte des Restes bernehmen
	cp   R2,R18	    	//	Rest (High) mit Divisor (High) vergleichen
	brlo div16_exec_2	//	Wenn Rest (High) kleiner ist, dann weiter 
	cp   R2,R18     
	brne div16_exec_3 	//	Wenn Rest (High) nicht gleich Divisor (High) ist, dann ist der Rest grer 
	cp   R3,R19			//	Wenn Rest (High) gleich Divisor (High) , dann Rest (low) und Divisor (Low) prfen
	brlo div16_exec_2	//	Wenn auch der Rest (Low) kleiner als der Divisor (Low) dann weiter
	rjmp div16_exec_3

div16_exec_2:
	
	//	Wenn Rest kleiner als Divisor...
	lsl  R1		//	eine null in das niederwertigste Bit des Quotienten schieben
	rol  R0
	rjmp div16_exec_4

div16_exec_3:

	//	Wenn Rest grer als Divisor...

	sub  R2,R18	//	Rest = Rest - Divisor
	sub  R3,R19
	sec				//	Carry Flag setzen
	rol	 R1		//	Mittels des gesetzten Carry Flags eine 1 in das niederwertigste Bit des Quotienten schieben
	rol  R0

div16_exec_4:
		
	dec	 R22		//	Zhler dekrementieren
	cpi	 R22,0x00	//	Zhler prfen ob bereits alle Berechnungen durchgefhrt wurden
	brne div16_exec_1	//	Wenn nein, dann weiter mit Berechnungen
	
	//	Register wiederherstellen
	pop R23
	pop R22

ret
